#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = require("commander");
const program = new commander_1.Command();
const replaceTextPieces = (text, pieces) => {
    const regex = new RegExp(Object.keys(pieces).join('|'), 'g');
    return text.replace(regex, (matched) => pieces[matched]);
};
program
    .version('1.0.0')
    .command('create <source> <destination>')
    .description('Creates a file or folder based on the <source> at <destination>')
    .option('-n, --name [name]', 'Changes the name of a file or directory', '')
    .option('-t, --templates-folder [path]', 'Path to templates directory', '__file-templates__')
    .option('-e, --encoding [encoding]', 'Changes the content encoding of the read files', 'utf-8')
    .option('-rn, --replace-names [names...]', 'Replaces the names of a file or directory', undefined)
    .option('-nb, --no-brackets', 'Makes brackets not required when using the --replace-names option')
    .option('-rc, --replace-content [content...]', 'Replaces parts of the contents of a file or files within a directory', undefined)
    .action((source, destination, options) => {
    const templatesDir = path_1.default.resolve(process.cwd(), options.templatesFolder);
    const sourcePath = path_1.default.resolve(templatesDir, source);
    const templatesDirExists = fs_1.default.existsSync(templatesDir);
    if (!templatesDirExists) {
        console.log(chalk_1.default.red(`The ${templatesDir} directory does not exist`));
        return;
    }
    const sourcePathExists = fs_1.default.existsSync(sourcePath);
    if (!sourcePathExists) {
        console.log(chalk_1.default.red(`${sourcePathExists} not found`));
        return;
    }
    const fsStatus = fs_1.default.lstatSync(sourcePath);
    const isSourcePathDirectory = fsStatus.isDirectory();
    const isSourcePathFile = fsStatus.isFile();
    const parsedNamesToReplace = options.replaceNames?.map((keyAndName) => {
        const [key, name] = keyAndName.split('=');
        return { key, name };
    });
    const isReplaceNamesOptionIncorrectly = parsedNamesToReplace?.some(({ key, name }) => !key || !name);
    if (isReplaceNamesOptionIncorrectly) {
        console.log(chalk_1.default.red(`The --replace-names option is incorrectly formatted`));
        console.log(chalk_1.default.green('Expected: '), chalk_1.default.green('-rn key1=name1 key2=name2'));
        console.log(chalk_1.default.redBright('Received: '), chalk_1.default.redBright(`-rn ${options.replaceNames}`));
        return;
    }
    const replaceContentObject = {};
    options.replaceContent?.forEach((keyAndText) => {
        const [key, text] = keyAndText.split('=');
        replaceContentObject[key] = text;
    });
    const isReplaceContentOptionIncorrectly = Object.entries(replaceContentObject).some(([key, text]) => !key || !text);
    if (isReplaceContentOptionIncorrectly) {
        console.log(chalk_1.default.red(`The --replace-content option is incorrectly formatted`));
        console.log(chalk_1.default.green('Expected: '), chalk_1.default.green('-rc key1=text1 key2=text2'));
        console.log(chalk_1.default.redBright('Received: '), chalk_1.default.redBright(`-rc ${options.replaceContent}`));
        return;
    }
    const canReplaceContent = Object.keys(replaceContentObject).length > 0;
    if (isSourcePathDirectory) {
        const directoryFiles = fs_1.default.readdirSync(sourcePath);
        if (directoryFiles.length === 0) {
            console.log(chalk_1.default.red(`No files found at ${sourcePath}`));
            return;
        }
        const directoryParts = source.split('/');
        const defaultDirectoryName = directoryParts[directoryParts.length - 1];
        let replacedDirectoryName = defaultDirectoryName;
        if (parsedNamesToReplace) {
            parsedNamesToReplace.forEach(({ key, name }) => {
                const keyToUse = options.brackets ? `[${key}]` : key;
                if (replacedDirectoryName.includes(keyToUse)) {
                    replacedDirectoryName = replacedDirectoryName.replace(keyToUse, name);
                }
            });
        }
        const directoryName = options.name || replacedDirectoryName;
        const destinationPath = path_1.default.resolve(process.cwd(), destination, directoryName);
        fs_1.default.mkdirSync(destinationPath);
        directoryFiles.forEach((fileName) => {
            const filePath = path_1.default.resolve(sourcePath, fileName);
            const fileContent = fs_1.default.readFileSync(filePath, options.encoding);
            const newFileContent = canReplaceContent
                ? replaceTextPieces(fileContent, replaceContentObject)
                : fileContent;
            let replacedFileName = fileName;
            if (parsedNamesToReplace) {
                parsedNamesToReplace.forEach(({ key, name }) => {
                    const keyToUse = options.brackets ? `[${key}]` : key;
                    if (replacedFileName.includes(keyToUse)) {
                        replacedFileName = replacedFileName.replace(keyToUse, name);
                    }
                });
            }
            const fileDestinationPath = path_1.default.resolve(process.cwd(), destinationPath, replacedFileName);
            fs_1.default.writeFileSync(fileDestinationPath, newFileContent);
            console.log(chalk_1.default.green(`File created at ${fileDestinationPath}`));
        });
    }
    else if (isSourcePathFile) {
        const sourceParts = source.split('/');
        const defaultFileName = sourceParts[sourceParts.length - 1];
        let replacedFileName = defaultFileName;
        if (parsedNamesToReplace) {
            parsedNamesToReplace.forEach(({ key, name }) => {
                const keyToUse = options.brackets ? `[${key}]` : key;
                if (replacedFileName.includes(keyToUse)) {
                    replacedFileName = replacedFileName.replace(keyToUse, name);
                }
            });
        }
        const fileName = options.name || replacedFileName;
        const destinationPath = path_1.default.resolve(process.cwd(), destination, fileName);
        const fileContent = fs_1.default.readFileSync(sourcePath, options.encoding);
        const newFileContent = canReplaceContent
            ? replaceTextPieces(fileContent, replaceContentObject)
            : fileContent;
        fs_1.default.writeFileSync(destinationPath, newFileContent);
        console.log(chalk_1.default.green(`File created at ${destinationPath}`));
    }
    else {
        console.log(chalk_1.default.red('The source can only be a file or a directory'));
    }
});
program.parse(process.argv);
